function [] = SOM(userM1,userM2,userInputData,userInitWeight,userBETAinit,userBETAfinal,userBETAtime,userSIGMAinit,userSIGMAfinal,userSIGMAtime,...
    userDistanceMeasure,userLearningRateDecay,userNeighborStrength,userShrinkingNeighborhood,userEpoch) 
% Self Organizing Maps

% X ... matrix of input vectors
% W ... array of weights
% i ... ith input neuron, 1<=i<=N
% N ... number of input neurons, dimensionality of input data
% m1m2 ... m1m2th output neuron, 1<=m1<=M1, 1<=m2<=M2
% M ... number of output neurons, M=M!*M2 (lattice M1xM2 neurons)
% M1,M2 ... size of Kohonen map
% I ... Ith input vector
% IV ... number of input vectors

%=========================================================================
% Structure of Network
%=========================================================================
% Output layer
%-------------
% type of output layer: 1D, 2D
% size of output layer:
M1 = userM1;
M2 = userM2;

% Input layer
%------------
% matrix of input data
X = imread(userInputData);
    original_image = imread(userInputData);
    % P1 P2 P3 P4 P5 P6 P7 P8 P9
    userInputData = dataPreparation(original_image);
    % normalization (<0,1>)
    X = userInputData/255;
% dimensionaly of input data:
IV = size(X,1);
N = size(X,2);


%=========================================================================
% Initialization
%=========================================================================
% epoch ... number of epochs
Epoch = userEpoch;

% InitWeight ... type of initialization weights:
%-----------------------------------------------
% 1) random small numbers
% 2) random numbers from the center of input space
% 3) randomly choose some input vectors
InitWeight = userInitWeight;

%--------------------------------------------------------------------------
% Learning parameter
%--------------------------------------------------------------------------

% setting of learning parameter
% -----------------------------
% beta ... lerning parameter, 0<=beta<=1, start: BETAinit (close to 1), end: BETAfinal (close to
% 0, >=0.01)
% BETAinit ... initial value of the parameter BETA
% BETAfinal ... final value of the parameter BETA
% BETAtime ... when the parameter BETA reaches BETAfinal value (1/2, 1/4...)
BETAinit = userBETAinit;
BETAfinal = userBETAfinal;
BETAtime = userBETAtime;
% the learning parameter must be in <0.01;1>
if BETAinit>1
    BETAinit = 1;
elseif BETAinit<0.01
    BETAinit = 0.01;
end
if BETAfinal<0.01
    BETAfinal = 0.01;
elseif BETAfinal>1
    BETAfinal = 1;
end
if BETAinit<BETAfinal
    BETAinit = 1;
    BETAfinal = 0.01;
end
if BETAtime>1
    BETAtime=1;
elseif BETAtime<0.01
    BETAtime=0.01;
end

% LearningRateDecay ... Learning rate decay
% -----------------------------------------
% 1) No decay (Constant learning rate)
% 2) Linear
% 3) Exponential
LearningRateDecay = userLearningRateDecay;

% Plot learning rate decay
% ------------------------
%plotLearningRateDecay(BETAinit,BETAfinal,BETAtime,LearningRateDecay,IV,Epoch)


%---------------------------------------------------------------------------
% Neigbourhood
%--------------------------------------------------------------------------

% setting of neighbourhood size 
% -----------------------------
% sigma ... radius of neighbourhood = neighbour size, start: SIGMAinit (size of output layer = SOM) ,
% end: SIGMAfinal (0)
% SIGMAinit ... initial radius of neighbourhood
% SIGMAfinal ... final radius of neighbourhood
% SIGMAtime ... when the neigbourhood radius reaches SIGMAfinal value (1/2, 1/4...)
SIGMAinit = userSIGMAinit;
SIGMAfinal = userSIGMAfinal;
SIGMAtime = userSIGMAtime;
% the neigbourhood size must be in <1,Mx>
Mx = [M1, M2];
Mx = max(Mx);
if SIGMAinit>Mx
    SIGMAinit = Mx;
elseif SIGMAinit<1
    SIGMAinit = 1;
end
if SIGMAfinal<1
    SIGMAfinal = 1;
elseif SIGMAfinal>Mx
    SIGMAfinal = Mx;
end
if SIGMAinit<SIGMAfinal
    SIGMAinit = Mx;
    SIGMAfinal = 1;
end
if SIGMAtime>1
    SIGMAtime=1;
elseif SIGMAtime<0.01
    SIGMAtime=0.01;
end

% ShrinkingNeighborhood ... shrinnking neighborhood size with iterations
% ----------------------------------------------------------------------
% 1) No decay (Constant neighborhood size)
% 2) Linear decay
% 3) Exponential decay
ShrinkingNeighborhood = userShrinkingNeighborhood;

% NeighborStrength ... Neighbor Strength function:
%-------------------------------------------------
% 1) Constant
% 2) Linear
% 3) Gaussian
% 4) Exponential
NeighborStrength = userNeighborStrength;

% Neigbourhood type
% ------------------
% 1) Linear
% 2) Square
% 3) Hexagonal

% Plot Neigbourhood settings
% --------------------------
%plotNeigbourhood(SIGMAinit,SIGMAfinal,SIGMAtime,ShrinkingNeighborhood,NeighborStrength,M1,M2,IV,Epoch)



[W] = initialization(M1,M2,N,X,InitWeight);

% visualization of input data and weights
%---------------------------------------
figureNumber = 1;
if N>1
    visualizationXW(N,X,W,figureNumber);
end

%=========================================================================
% Learning
%=========================================================================
% DistanceMeasure ... distance measures:
%---------------------------------------
% 1) Euclidean distance
% 2) Correlation
% 3) Direction cosines
% 4) City block distance
DistanceMeasure = userDistanceMeasure;

[Wvect,con,classDist,classN,classX] = learning(Epoch,DistanceMeasure,LearningRateDecay,NeighborStrength,ShrinkingNeighborhood,IV,X,M1,M2,W, BETAinit,BETAfinal,BETAtime,SIGMAinit,SIGMAfinal,SIGMAtime,N);

% determination of weight matrix
%---------------------------------
[num,ind] = min(con);
disp('Epocha s nejmensim con'); disp(ind);
W= Wvect(:,:,:,ind);

%=========================================================================
% Results
%=========================================================================
% visualization of SOM convergence
%---------------------------------
figure(2)
plot(1: Epoch, (con/(M1*M2))); % normalizace na odstraneni vlivu velikosti vystupni vrstvy (velikost M1, M2)
    xlabel('Epochs'); ylabel('Mean distance');
    title('SOM convergence');
    grid on;

% visualization of input data and final weights
%-----------------------------------------
figureNumber = 3;
visualizationXW(N,X,W,figureNumber);
set(gcf,'Name', 'Final weight vectors superimposed on data');

% visualization of input data and weights
%-----------------------------------------
% figure(4);
% RI1 = repmat(1:M1,[M2,1])';
% RI1 = RI1(:);
% RI2 = repmat(1:M2,[M1,1]);
% RI2 = RI2(:);
% classDist = classDist(:);
% plot3(RI1,RI2,classDist,'+');
%  xlabel('m1'); ylabel('m2'); zlabel('ClassDist');
% for i=1:M
%     s =['class [',num2str(RI1(i)),',',num2str(RI2(i)),']'];
%     text(RI1(i),RI2(i),classDist(i),s);
% end
% grid on;

if M1==1
    figure;
        bar(classN(:,:,end)); title('Number of input vectors belonging to class');
        xlabel('Class'); ylabel('Number of input vectors');
    figure;
        bar(classDist(:,:,end)); title('Errors in classes'); 
        xlabel('Class'); ylabel('Error');
else
    figure;
        bar3(classN(:,:,end)); title('Number of input vectors belonging to class');
        xlabel('Class'); ylabel('Number of input vectors');
    figure;
        bar3(classDist(:,:,end)); title('Errors in classes');  
        xlabel('Class'); ylabel('Error');
end

% image_original=imread('pokus.jpg'); image_original = rgb2ind(image_original,gray);
%     figure; imshow(image_original); axis image; colormap(gray); 
% C = classX(:,2,end);
% C = reshape(C,size(image_original,2)-2,size(image_original,1)-2)';
%     figure; imagesc(C); % Create a colored plot of the matrix values
%         colormap(flipud(gray));

% % assignment
% Yfilename = Yf;
% %Yfilename='Y_backassignment';
% %Yfilename='Y_v3';
% Y = assignment(IV,X,M1,W,Yfilename);
