function [W] = adjustment(LearningRateDecay,NeighborStrength,ShrinkingNeighborhood,BETAinit,BETAfinal,BETAtime,SIGMAinit,SIGMAfinal,SIGMAtime,iv,e,Epoch,IV,x,M1,M2,N,m1,m2,W)
% adjustment of weights

%==========================================================================
% adjustment of learning parameter
%==========================================================================

% LearningRateDecay ... Learning rate decay:
if LearningRateDecay==1
    % Constant
    % beta = BETA
    beta = BETAinit;
    BETAfinal = BETAinit;
    % BETAtime = 0;
    
elseif LearningRateDecay==2
    % Linear
    % beta = BETA*(1-t*k/T)
    k = (1-BETAfinal/BETAinit)/BETAtime;
    beta = BETAinit*(1-((e-1)*IV+iv)*k/(IV*Epoch));
    
elseif LearningRateDecay==3
    % Exponential
    % beta = BETA*exp(-t*k/T)
    k = -log(BETAfinal/BETAinit)/BETAtime;
    beta = BETAinit*exp(-((e-1)*IV+iv)*k/(IV*Epoch));
end

if (beta<BETAfinal)
    beta = BETAfinal;
end

%==========================================================================
% adjustment of neighbor strength
%==========================================================================

% ShrinkingNeighborhood ... shrinnking neighborhood size with iterations
% ----------------------------------------------------------------------
if ShrinkingNeighborhood==1
    % 1) Constant
    % sigma = SIGMA
    sigma = SIGMAinit;
    SIGMAfinal = SIGMAinit;
    % SIGMAtime = 0;    
    
elseif ShrinkingNeighborhood==2
    % 2) Linear
    %sigma = SIGMA*(1-t*k/T)
    k = (1-SIGMAfinal/SIGMAinit)/SIGMAtime;
    sigma = SIGMAinit*(1-((e-1)*IV+iv)*k/(IV*Epoch));

elseif ShrinkingNeighborhood==3
    % 3) Exponential decay
    %sigma = SIGMA*Exp(-t*k/T)
    k = -log(SIGMAfinal/SIGMAinit)/SIGMAtime;
    sigma = SIGMAinit*exp(-((e-1)*IV+iv)*k/(IV*Epoch));
end

if (sigma<SIGMAfinal)
    sigma = SIGMAfinal;
end

% Euclidean distance d
% d=rj-ri=sqr((rj1^-ri1)^2+(rj2-ri2)^2)
% columns
RI1 = repmat(1:M1,[M2,1])';
% rows
RI2 = repmat(1:M2,[M1,1]);
d = (m1-RI1).^2+(m2-RI2).^2;
d = repmat(d,[1,1,N]);

% NeighborStrength ... Neighbor Strength function:
%-------------------------------------------------
if NeighborStrength==1
    % Constant
    NS = 1;
    
elseif NeighborStrength==2
    % Linear
    %NS = 1-d/sigma;
    if sigma==0
        NS = zeros(size(d));
    else
        NS = 1-(sqrt(d)/sigma);
    end
        
elseif NeighborStrength==3
    % Gaussian
    % NS = exp(-dij^2/2/sigma^2)
    % NS = exp(-dij^2/2/10/sigma^2)
    if sigma==0
        NS = zeros(size(d));
    else
        NS = exp(-d/(2/10*sigma^2));
    end
    
elseif NeighborStrength==4
    % Exponential
    % NS = exp(-k*dij)
    if sigma==0
        NS = zeros(size(d));
    else    
        k = -log(0.001)/sigma;
        NS = exp(-k*d);
    end
end

NS=(NS>0).*NS;
%==========================================================================
% adjustment of weights
%==========================================================================

% ns ... learning parameter (beta) * neighbor strength (NS)
ns = beta.*NS;

% matrix operations and calculations
xM = repmat(x,[M1,1,M2]);
xM = permute(xM,[1,3,2]);
% beta = repmat(beta,[M1,M2,N]);
% W' = W + beta*NS*(x-W)
W = W + ns.*(xM-W);

end