function [W] = initialization(M1,M2,N,X,InitWeight)
% Initialization

% =========================================================================
% Setting of weights
% =========================================================================
% InitWeight ... type of initialization weights:
% 1) random small numbers
% 2) random numbers from the center of input space
% 3) randomly choose some input vectors

if InitWeight==1
    % setting of weights - random small numbers
    % -----------------------------------------
    % wSD ... standard deviation
    % wMEAN ... mean
    % W ... matrix of weights, [M1,M2,N]
    % k ... coeficient of smallness
    k = 10;
    wSD = std(X); % vector of std of particular features
    wSD = permute(wSD,[1,3,2]);
    wSD = wSD/k;
    wMEAN = mean(X);
    wMEAN = permute(wMEAN, [1,3,2]);
    wMEAN = wMEAN/k;
    W = repmat(wSD,[M1,M2,1]).*repmat(randn(M1,M2),[1,1,N]) + repmat(wMEAN,[M1,M2,1]);
    
elseif InitWeight==2
    % setting of weights - random numbers from the center of input space
    % ------------------------------------------------------------------
    % wSD ... standard deviation of input data
    % wMEAN ... center of input space [1,N]
    % W ... matrix of weights, [M1,M2,N]
    % k ... coeficient of size of "center" of input data
    k = 10;
    wSD = std(X); % vector of std of particular features
    wSD = permute(wSD,[1,3,2]);
    wSD = wSD/k;
    wMEAN = mean(X);
    wMEAN = permute(wMEAN, [1,3,2]);
    W = repmat(wSD,[M1,M2,1]).*repmat(randn(M1,M2),[1,1,N]) + repmat(wMEAN,[M1,M2,1]);
    
elseif InitWeight==3
    % setting of weights - randomly choose some input vectors
    % -------------------------------------------------------
    % speed up learning
    % W ... matrix of weights, [M1,M2,N]
    IV = length(X);
    W = randi(IV,[M1,M2]);
    W = X(W,:);
    W = W(:);
    W = reshape(W, [M1,M2,N]);
end


% =========================================================================
% setting of neighbourhood 
% =========================================================================
% selection of neighborhood geometry
% 1) linear
% 2) square
% 3) hexagnal

end